@@ -71,10 +71,13 @@ globalVariables(c(".",
                   "more_than_episode_ago",
                   "n",
                   "name",
+                  "new",
                   "observations",
                   "observed",
                   "official",
+                  "old",
                   "other_pat_or_mo",
+                  "package_v",
                   "Pasted",
                   "patient_id",
                   "phylum",
